]> the.earth.li Git - onak.git/blobdiff - keydb.c
Move mailsync functionality to the database backends.
[onak.git] / keydb.c
diff --git a/keydb.c b/keydb.c
index 3fe3d34d85afe8632e9426b2e08a3ea23bdb897e..b8bd316e8e3dcf01fbb4e0f1efc02efe7eb7997e 100644 (file)
--- a/keydb.c
+++ b/keydb.c
@@ -3,7 +3,7 @@
  *
  * Jonathan McDowell <noodles@earth.li>
  *
- * Copyright 2002 Project Purple
+ * Copyright 2002-2004 Project Purple
  */
 
 /**
@@ -22,6 +22,7 @@
 #include "keyid.h"
 #include "keystructs.h"
 #include "mem.h"
+#include "merge.h"
 #include "parsekey.h"
 
 #ifdef NEED_KEYID2UID
@@ -61,11 +62,13 @@ char *keyid2uid(uint64_t keyid)
 /**
  *     getkeysigs - Gets a linked list of the signatures on a key.
  *     @keyid: The keyid to get the sigs for.
+ *     @revoked: Is the key revoked?
  *
  *     This function gets the list of signatures on a key. Used for key 
*     indexing and doing stats bits.
  */
-struct ll *getkeysigs(uint64_t keyid)
+struct ll *getkeysigs(uint64_t keyid, bool *revoked)
 {
        struct ll *sigs = NULL;
        struct openpgp_signedpacket_list *uids = NULL;
@@ -77,6 +80,9 @@ struct ll *getkeysigs(uint64_t keyid)
                for (uids = publickey->uids; uids != NULL; uids = uids->next) {
                        sigs = keysigs(sigs, uids->sigs);
                }
+               if (revoked != NULL) {
+                       *revoked = (publickey->revocations != NULL);
+               }
                free_publickey(publickey);
        }
 
@@ -95,28 +101,24 @@ struct ll *getkeysigs(uint64_t keyid)
 struct ll *cached_getkeysigs(uint64_t keyid)
 {
        struct stats_key *key = NULL;
+       struct stats_key *signedkey = NULL;
+       struct ll        *cursig = NULL;
+       bool              revoked = false;
 
        if (keyid == 0)  {
                return NULL;
        }
 
-       key = findinhash(keyid);
-       if (key == NULL) {
-               key = malloc(sizeof(*key));
-               if (key != NULL) {
-                       key->keyid = keyid;
-                       key->colour = 0;
-                       key->parent = 0;
-                       key->sigs = NULL;
-                       key->gotsigs = false;
-                       addtohash(key);
-               } else {
-                       perror("cached_getkeysigs()");
-                       return NULL;
-               }
-       }
+       key = createandaddtohash(keyid);
+
        if (key->gotsigs == false) {
-               key->sigs = getkeysigs(key->keyid);
+               key->sigs = getkeysigs(key->keyid, &revoked);
+               key->revoked = revoked;
+               for (cursig = key->sigs; cursig != NULL;
+                               cursig = cursig->next) {
+                       signedkey = (struct stats_key *) cursig->object;
+                       signedkey->signs = lladd(signedkey->signs, key);
+               }
                key->gotsigs = true;
        }
 
@@ -149,3 +151,75 @@ uint64_t getfullkeyid(uint64_t keyid)
        return keyid;
 }
 #endif
+
+#ifdef NEED_UPDATEKEYS
+/**
+ *     update_keys - Takes a list of public keys and updates them in the DB.
+ *     @keys: The keys to update in the DB.
+ *     @sendsync: Should we send a sync mail to our peers.
+ *
+ *     Takes a list of keys and adds them to the database, merging them with
+ *     the key in the database if it's already present there. The key list is
+ *     update to contain the minimum set of updates required to get from what
+ *     we had before to what we have now (ie the set of data that was added to
+ *     the DB). Returns the number of entirely new keys added.
+ */
+int update_keys(struct openpgp_publickey **keys, bool sendsync)
+{
+       struct openpgp_publickey *curkey = NULL;
+       struct openpgp_publickey *oldkey = NULL;
+       struct openpgp_publickey *prev = NULL;
+       int newkeys = 0;
+       bool intrans;
+
+       for (curkey = *keys; curkey != NULL; curkey = curkey->next) {
+               intrans = starttrans();
+               logthing(LOGTHING_INFO,
+                       "Fetching key 0x%llX, result: %d",
+                       get_keyid(curkey),
+                       fetch_key(get_keyid(curkey), &oldkey, intrans));
+
+               /*
+                * If we already have the key stored in the DB then merge it
+                * with the new one that's been supplied. Otherwise the key
+                * we've just got is the one that goes in the DB and also the
+                * one that we send out.
+                */
+               if (oldkey != NULL) {
+                       merge_keys(oldkey, curkey);
+                       if (curkey->revocations == NULL &&
+                                       curkey->uids == NULL &&
+                                       curkey->subkeys == NULL) {
+                               if (prev == NULL) {
+                                       *keys = curkey->next;
+                               } else {
+                                       prev->next = curkey->next;
+                                       curkey->next = NULL;
+                                       free_publickey(curkey);
+                                       curkey = prev;
+                               }
+                       } else {
+                               prev = curkey;
+                               logthing(LOGTHING_INFO,
+                                       "Merged key; storing updated key.");
+                               store_key(oldkey, intrans, true);
+                       }
+                       free_publickey(oldkey);
+                       oldkey = NULL;
+               } else {
+                       logthing(LOGTHING_INFO,
+                               "Storing completely new key.");
+                       store_key(curkey, intrans, false);
+                       newkeys++;
+               }
+               endtrans();
+               intrans = false;
+       }
+
+       if (sendsync && keys != NULL) {
+               sendkeysync(keys);
+       }
+
+       return newkeys;
+}
+#endif /* NEED_UPDATEKEYS */
@@ -61,11 +62,13 @@ char *keyid2uid(uint64_t keyid)
  *
  *     This function gets the list of signatures on a key. Used for key 
- *     indexing and doing stats bits.
+ *     indexing and doing stats bits. If revoked is non-NULL then if the key
+ *     is revoked it's set to true.
  */